import Glib from "gi://GLib";
import Gio from "gi://Gio";
import Shell from "gi://Shell";
import { fileExists, readFile, uniqueId } from "./util.js";
export default class VSCodeSearchProvider {
    workspaces = {};
    extension;
    app = null;
    appInfo;
    constructor(extension) {
        this.extension = extension;
        this._findApp();
        const codeConfig = this._getConfig();
        if (!codeConfig) {
            console.error("Failed to read vscode storage.json");
            return;
        }
        const paths = Object.keys(codeConfig.profileAssociations.workspaces).sort();
        for (const path of paths) {
            if (path.startsWith("vscode-remote://dev-container")) {
                continue;
            }
            const name = path.split("/").pop();
            this.workspaces[uniqueId()] = { name, path: path.replace("file://", "") };
        }
        this.appInfo = this.app?.appInfo;
    }
    _getConfig() {
        const configDirs = [
            Glib.get_user_config_dir(),
            `${Glib.get_home_dir()}/.var/app`,
        ];
        const appDirs = [
            // XDG_CONFIG_DIRS
            "Code",
            "Code - Insiders",
            "VSCodium",
            "VSCodium - Insiders",
            // Flatpak
            "com.vscodium.codium/config/VSCodium",
            "com.vscodium.codium-insiders/config/VSCodium - Insiders",
        ];
        for (const configDir of configDirs) {
            for (const appDir of appDirs) {
                const path = `${configDir}/${appDir}/User/globalStorage/storage.json`;
                if (!fileExists(path)) {
                    continue;
                }
                const storage = readFile(path);
                if (storage) {
                    return JSON.parse(storage);
                }
            }
        }
    }
    _findApp() {
        const ids = [
            "code",
            "code-insiders",
            "code-oss",
            "com.vscodium.codium",
            "com.vscodium.codium-insiders",
        ];
        for (let i = 0; !this.app && i < ids.length; i++) {
            this.app = Shell.AppSystem.get_default().lookup_app(ids[i] + ".desktop");
        }
        if (!this.app) {
            console.error("Failed to find vscode application");
        }
    }
    activateResult(result) {
        if (this.app) {
            this.app?.app_info.launch([Gio.file_new_for_path(this.workspaces[result].path)], null);
        }
    }
    filterResults(results, maxResults) {
        return results.slice(0, maxResults);
    }
    async getInitialResultSet(terms) {
        return Object.keys(this.workspaces).filter((id) => this.workspaces[id].name.includes(terms.join("")));
    }
    async getSubsearchResultSet(previousResults, terms) {
        return previousResults.filter((id) => this.workspaces[id].name.includes(terms.join("")));
    }
    async getResultMetas(ids) {
        return ids.map((id) => ({
            id,
            name: this.workspaces[id].name,
            description: this.workspaces[id].path,
            createIcon: (size) => this.app?.create_icon_texture(size),
        }));
    }
}
